@@ -70,7 +70,7 @@ class ArgType:
 	parameter."""
 	raise RuntimeError, "write_param not implemented for %s" % \
               self.__class__.__name__
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
 	"""Adds a variable named ret of the return type to
 	info.varlist, and add any required code to info.codeafter to
 	convert the return value to a python object."""
@@ -78,7 +78,7 @@ class ArgType:
               self.__class__.__name__
 
 class NoneArg(ArgType):
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.codeafter.append('    Py_INCREF(Py_None);\n' +
                               '    return Py_None;')
 
@@ -94,14 +94,8 @@ class StringArg(ArgType):
             info.add_parselist('z', ['&' + pname], [pname])
 	else:
             info.add_parselist('s', ['&' + pname], [pname])
-    def write_return(self, ptype, info):
-        if ptype[:6] == 'const-' or ptype in ('static_string',):
-	    info.varlist.add('const gchar', '*ret')
-            info.codeafter.append('    if (ret)\n' +
-                                  '        return PyString_FromString(ret);\n'+
-                                  '    Py_INCREF(Py_None);\n' +
-                                  '    return Py_None;')
-	else:
+    def write_return(self, ptype, ownsreturn, info):
+        if ownsreturn:
 	    # have to free result ...
 	    info.varlist.add('gchar', '*ret')
             info.codeafter.append('    if (ret) {\n' +
@@ -111,6 +105,12 @@ class StringArg(ArgType):
                                   '    }\n' +
                                   '    Py_INCREF(Py_None);\n' +
                                   '    return Py_None;')
+	else:
+	    info.varlist.add('const gchar', '*ret')
+            info.codeafter.append('    if (ret)\n' +
+                                  '        return PyString_FromString(ret);\n'+
+                                  '    Py_INCREF(Py_None);\n' +
+                                  '    return Py_None;')
 
 class UCharArg(ArgType):
     # allows strings with embedded NULLs.
@@ -136,9 +136,45 @@ class CharArg(ArgType):
 	    info.varlist.add('char', pname)
 	info.arglist.append(pname)
         info.add_parselist('c', ['&' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
 	info.varlist.add('gchar', 'ret')
         info.codeafter.append('    return PyString_FromStringAndSize(&ret, 1);')
+class GUniCharArg(ArgType):
+    param_tmpl = ('    if (py_%(name)s[1] != 0) {\n'
+                  '        PyErr_SetString(PyExc_TypeError, "%(name)s should be a 1 character unicode string");\n'
+                  '        return NULL;\n'
+                  '    }\n'
+                  '    %(name)s = (gunichar)py_%(name)s[0];\n')
+    dflt_tmpl = ('    if (py_%(name)s != NULL) {\n'
+                 '        if (py_%(name)s[1] != 0) {\n'
+                 '            PyErr_SetString(PyExc_TypeError, "%(name)s should be a 1 character unicode string");\n'
+                 '            return NULL;\n'
+                 '        }\n'
+                 '        %(name)s = (gunichar)py_%(name)s[0];\n'
+                 '     }\n')
+    ret_tmpl = ('#if !defined(Py_UNICODE_SIZE) || Py_UNICODE_SIZE == 2\n'
+                '    if (ret > 0xffff) {\n'
+                '        PyErr_SetString(PyExc_RuntimeError, "returned character can not be represented in 16-bit unicode");\n'
+                '        return NULL;\n'
+                '    }\n'
+                '#endif\n'
+                '    py_ret = (Py_UNICODE)ret;\n'
+                '    return PyUnicode_FromUnicode(&py_ret, 1);\n')
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+	if pdflt:
+	    info.varlist.add('gunichar', pname + " = '" + pdflt + "'")
+            info.codebefore.append(self.dflt_tmpl % {'name':pname})
+	else:
+	    info.varlist.add('gunichar', pname)
+            info.codebefore.append(self.param_tmpl % {'name':pname})
+        info.varlist.add('Py_UNICODE', '*py_' + pname + ' = NULL')
+	info.arglist.append(pname)
+        info.add_parselist('u', ['&py_' + pname], [pname])
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add('gunichar', 'ret')
+        info.varlist.add('Py_UNICODE', 'py_ret')
+        info.codeafter.append(self.ret_tmpl)
+        
 
 class IntArg(ArgType):
     def write_param(self, ptype, pname, pdflt, pnull, info):
@@ -148,10 +184,37 @@ class IntArg(ArgType):
 	    info.varlist.add('int', pname)
 	info.arglist.append(pname)
         info.add_parselist('i', ['&' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('int', 'ret')
         info.codeafter.append('    return PyInt_FromLong(ret);')
 
+class ULongArg(ArgType):
+    dflt = ('    if (py_%(name)s && PyLong_Check(py_%(name)s))\n'
+            '        %(name)s = PyLong_AsUnsignedLong(py_%(name)s);\n'
+            '    else if (py_%(name)s) {\n'
+            '        PyErr_SetString(PyExc_TypeError, "%(name)s must be an int");\n'
+            '        return NULL;\n'
+            '    }\n')
+    before = ('    if (py_%(name)s && PyLong_Check(py_%(name)s))\n'
+            '        %(name)s = PyLong_AsUnsignedLong(py_%(name)s);\n'
+            '    else {\n'
+            '        PyErr_SetString(PyExc_TypeError, "%(name)s must be an int");\n'
+            '        return NULL;\n'
+            '    }\n')
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+        if pdflt:
+            info.varlist.add('unsigned int', pname + ' = ' + pdflt)
+            info.codebefore.append(self.dflt % {'name':pname})            
+        else:
+            info.varlist.add('unsigned int', pname)
+            info.codebefore.append(self.before % {'name':pname})            
+        info.varlist.add('PyObject', "*py_" + pname + ' = NULL')
+        info.arglist.append(pname)
+        info.add_parselist('O', ['&py_' + pname], [pname])
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add('unsigned int', 'ret')
+        info.codeafter.append('    return PyLong_FromUnsignedLong(ret);')
+
 class DoubleArg(ArgType):
     def write_param(self, ptype, pname, pdflt, pnull, info):
 	if pdflt:
@@ -160,7 +223,7 @@ class DoubleArg(ArgType):
 	    info.varlist.add('double', pname)
 	info.arglist.append(pname)
         info.add_parselist('d', ['&' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('double', 'ret')
         info.codeafter.append('    return PyFloat_FromDouble(ret);')
 
@@ -203,7 +266,7 @@ class FileArg(ArgType):
 		info.varlist.add('PyObject', '*' + pname)
 		info.arglist.append('PyFile_AsFile(' + pname + ')')
             info.add_parselist('O!', ['&PyFile_Type', '&' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
 	info.varlist.add('FILE', '*ret')
         info.codeafter.append('    if (ret)\n' +
                               '        return PyFile_FromFile(ret, "", "", fclose);\n' +
@@ -226,7 +289,7 @@ class EnumArg(ArgType):
                                              'name': pname})
 	info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname]);
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('gint', 'ret')
         info.codeafter.append('    return PyInt_FromLong(ret);')
 
@@ -249,7 +312,7 @@ class FlagsArg(ArgType):
                                             'name':pname})
 	info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('guint', 'ret')
         info.codeafter.append('    return PyInt_FromLong(ret);')
 
@@ -316,11 +379,17 @@ class ObjectArg(ArgType):
                                                      'type':self.objname}) 
 		info.arglist.append('%s(%s->obj)' % (self.cast, pname))
                 info.add_parselist('O', ['&' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         if ptype[-1] == '*': ptype = ptype[:-1]
         info.varlist.add(ptype, '*ret')
-        info.codeafter.append('    /* pygobject_new handles NULL checking */\n' +
-                              '    return pygobject_new((GObject *)ret);')
+        if ownsreturn:
+            info.varlist.add('PyObject', '*py_ret')
+            info.codeafter.append('    py_ret = pygobject_new((GObject *)ret;\n'
+                                  '    g_object_unref(ret);\n'
+                                  '    return py_ret;')
+        else:
+            info.codeafter.append('    /* pygobject_new handles NULL checking */\n' +
+                                  '    return pygobject_new((GObject *)ret);')
 
 class BoxedArg(ArgType):
     # haven't done support for default args.  Is it needed?
@@ -354,15 +423,19 @@ class BoxedArg(ArgType):
                                                  'typecode': self.typecode})
         info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname])
-    def write_return(self, ptype, info):
+    ret_tmpl = '    /* pyg_boxed_new handles NULL checking */\n' \
+               '    return pyg_boxed_new(%(typecode)s, %(ret)s, %(copy)s, TRUE);'
+    def write_return(self, ptype, ownsreturn, info):
         if ptype[-1] == '*':
             info.varlist.add(self.typename, '*ret')
-            info.codeafter.append('    /* pyg_boxed_new handles NULL checking */\n' +
-                                  '    return pyg_boxed_new(' + self.typecode + ', ret, TRUE, TRUE);')
+            ret = 'ret'
         else:
             info.varlist.add(self.typename, 'ret')
-            info.codeafter.append('    /* pyg_boxed_new handles NULL checking */\n' +
-                                  '    return pyg_boxed_new(' + self.typecode + ', &ret, TRUE, TRUE);')
+            ret = '&ret'
+        info.codeafter.append(self.ret_tmpl %
+                              { 'typecode': self.typecode,
+                                'ret': ret,
+                                'copy': ownsreturn and 'TRUE' or 'FALSE'})
 
 class CustomBoxedArg(ArgType):
     # haven't done support for default args.  Is it needed?
@@ -391,7 +464,7 @@ class CustomBoxedArg(ArgType):
 	    info.varlist.add('PyObject', '*' + pname)
 	    info.arglist.append(self.getter + '(' + pname + ')')
             info.add_parselist('O!', ['&' + self.pytype, '&' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add(ptype[:-1], '*ret')
         info.codeafter.append('    if (ret)\n' +
                               '        return ' + self.new + '(ret);\n' +
@@ -430,7 +503,7 @@ class PointerArg(ArgType):
                                                  'typecode': self.typecode})
         info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         if ptype[-1] == '*':
             info.varlist.add(self.typename, '*ret')
             info.codeafter.append('    /* pyg_pointer_new handles NULL checking */\n' +
@@ -450,7 +523,7 @@ class AtomArg(IntArg):
 	info.codebefore.append(self.atom % {'name': pname})
 	info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('GdkAtom', 'ret')
         info.codeafter.append('    return PyGdkAtom_New(ret);')
 
@@ -463,7 +536,7 @@ class GTypeArg(ArgType):
 	info.codebefore.append(self.gtype % {'name': pname})
 	info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname])
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('GType', 'ret')
         info.codeafter.append('    return pyg_type_wrapper_new(ret);')
 
@@ -503,16 +576,23 @@ class GtkTreePathArg(ArgType):
 	    info.arglist.append(pname)
             info.add_parselist('O!', ['&PyTuple_Type', '&py_' + pname],[pname])
         info.codeafter.append(self.freepath % {'name': pname})
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('GtkTreePath', '*ret')
-        info.codeafter.append('    if (ret) {\n' +
-                              '        PyObject *py_ret = pygtk_tree_path_to_pyobject(ret);\n' +
-                              '        gtk_tree_path_free(ret);\n' +
-                              '        return py_ret;\n' +
-                              '    }\n' +
-                              '    Py_INCREF(Py_None);\n' +
-                              '    return Py_None;')
-													       
+        if ownsreturn:
+            info.codeafter.append('    if (ret) {\n'
+                                  '        PyObject *py_ret = pygtk_tree_path_to_pyobject(ret);\n'
+                                  '        gtk_tree_path_free(ret);\n'
+                                  '        return py_ret;\n'
+                                  '    }\n'
+                                  '    Py_INCREF(Py_None);\n'
+                                  '    return Py_None;')
+        else:
+            info.codeafter.append('    if (ret) {\n'
+                                  '        PyObject *py_ret = pygtk_tree_path_to_pyobject(ret);\n'
+                                  '        return py_ret;\n'
+                                  '    }\n'
+                                  '    Py_INCREF(Py_None);\n'
+                                  '    return Py_None;')
 						       
 class GdkRectanglePtrArg(ArgType):
     null = ('    if (PyTuple_Check(py_%(name)s)) {\n'
@@ -536,10 +616,10 @@ class GdkRectanglePtrArg(ArgType):
 	info.codebefore.append(self.null % {'name':  pname})
 
 class GdkRectangleArg(ArgType):
-    def write_return(self, ptype, info):
+    def write_return(self, ptype, ownsreturn, info):
 	info.varlist.add('GdkRectangle', 'ret')
 	info.codeafter.append('    return pyg_boxed_new(GDK_RECTANGLE, ret, TRUE, TRUE);')
-	
+
 class ArgMatcher:
     def __init__(self):
 	self.argtypes = {}
@@ -578,7 +658,12 @@ class ArgMatcher:
         self.register('const-'+ptype+'*', arg)
 
     def get(self, ptype):
-	return self.argtypes[ptype]
+        try:
+            return self.argtypes[ptype]
+        except KeyError:
+            if ptype[:8] == 'GdkEvent' and ptype[-1] == '*':
+                return self.argtypes['GdkEvent*']
+            raise
     def object_is_a(self, otype, parent):
         if otype == None: return 0
         if otype == parent: return 1
@@ -609,6 +694,9 @@ matcher.register('char', arg)
 matcher.register('gchar', arg)
 matcher.register('guchar', arg)
 
+arg = GUniCharArg()
+matcher.register('gunichar', arg)
+
 arg = IntArg()
 matcher.register('int', arg)
 matcher.register('gint', arg)
@@ -618,17 +706,26 @@ matcher.register('gshort', arg)
 matcher.register('gushort', arg)
 matcher.register('long', arg)
 matcher.register('glong', arg)
-matcher.register('gulong', arg)
 matcher.register('gboolean', arg)
 
 matcher.register('guint8', arg)
 matcher.register('gint8', arg)
 matcher.register('guint16', arg)
 matcher.register('gint16', arg)
-matcher.register('guint32', arg)
 matcher.register('gint32', arg)
 matcher.register('GtkType', arg)
 
+# If the system maxint is smaller than unsigned int, we need to use
+# Long objects with PyLong_AsUnsignedLong
+if sys.maxint >= (1L << 32):
+    matcher.register('guint32', arg)
+else:
+    arg = ULongArg()
+    matcher.register('guint32', arg)
+
+arg = ULongArg()
+matcher.register('gulong', arg)
+
 arg = DoubleArg()
 matcher.register('double', arg)
 matcher.register('gdouble', arg)
